# Generated by Django 5.1.6 on 2025-02-23 16:57

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_login_history2', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='login',
            name='logout_at',
            field=models.DateTimeField(blank=True, null=True, verbose_name='logout at'),
        ),
        migrations.AlterField(
            model_name='login',
            name='city',
            field=models.CharField(blank=True, max_length=80, verbose_name='city'),
        ),
        migrations.AlterField(
            model_name='login',
            name='country_code',
            field=models.CharField(blank=True, max_length=2, verbose_name='country code'),
        ),
        migrations.AlterField(
            model_name='login',
            name='country_name',
            field=models.CharField(blank=True, max_length=80, verbose_name='country name'),
        ),
        migrations.AlterField(
            model_name='login',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='created at'),
        ),
        migrations.AlterField(
            model_name='login',
            name='currency',
            field=models.CharField(blank=True, max_length=5, verbose_name='currency'),
        ),
        migrations.AlterField(
            model_name='login',
            name='ip',
            field=models.GenericIPAddressField(db_index=True, verbose_name='ip address'),
        ),
        migrations.AlterField(
            model_name='login',
            name='ip_info',
            field=models.JSONField(default=dict, verbose_name='ip info'),
        ),
        migrations.AlterField(
            model_name='login',
            name='region',
            field=models.CharField(blank=True, max_length=80, verbose_name='region'),
        ),
        migrations.AlterField(
            model_name='login',
            name='region_code',
            field=models.CharField(blank=True, max_length=10, verbose_name='region code'),
        ),
        migrations.AlterField(
            model_name='login',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
        migrations.AlterField(
            model_name='login',
            name='user_agent',
            field=models.TextField(verbose_name='user agent'),
        ),
    ]
